﻿rsync-now
=========

`rsync-now` est un outil basé sur `rsync`, il vous aidera à mettre en place vos sauvegardes.

Il est livré avec un outil de configuration : `rsync-now-config`.

`rsync-now` permet de sauvegarder vos données localement ou sur un serveur distant en utilisant le protocole `ssh`.


Installation
------------

Pour télécharger le projet et réaliser l’installation vous devez utiliser les commandes suivantes :

```bash
git clone --recurse-submodules https://git.cclaude.rocks/tools/rsync-now.git
cd rsync-now
sudo bash rsync-now-install.sh
```

### Options d’installation

Vous pouvez obtenir l’aide en utilisant :

```bash
bash ./prepare-raspbian-sd-install.sh --help
```

| Option                            | Description                   |
|-----------------------------------|-------------------------------|
| -S, --skip-update                 | To skip update before install required packets |
| -P PMEXE, --package-manager PMEXE | Force package manager binary name |


Configuration rapide
--------------------

Vous devez d’abord créer la configuration en utilisant `rsync-now-config` :

```bash
rsync-now-config -new "$( xdg-user-dir DOCUMENTS )" '/path/to/external_drive' 'Save to USB Drive'
```

Utilisation simple :

```bash
rsync-now
```


Exemple de configuration
------------------------

Dans un premier temps, vous devez définir les dossiers que vous souhaitez sauvegarder,
voici un exemple de code qui devrait fonctionner sur la plupart des distributions Linux.

```bash
rsync-now-config -C DEFAULT --new '/home/claude/Documents'   '${backups_dir}/DOCUMENTS'      'Backup DOCUMENTS'
rsync-now-config -C DEFAULT --new '/home/claude/Downloads'   '${backups_dir}/DOWNLOAD'       'Backup DOWNLOAD'
rsync-now-config -C DEFAULT --new '/home/claude/Music'       '${backups_dir}/MUSIC'          'Backup MUSIC'
rsync-now-config -C DEFAULT --new '/home/claude/Pictures'    '${backups_dir}/PICTURES'       'Backup PICTURES'
rsync-now-config -C DEFAULT --new '/home/claude/Public'      '${backups_dir}/PUBLICSHARE'    'Backup PUBLICSHARE'
rsync-now-config -C DEFAULT --new '/home/claude/Templates'   '${backups_dir}/TEMPLATES'      'Backup TEMPLATES'
rsync-now-config -C DEFAULT --new '/home/claude/Videos'      '${backups_dir}/VIDEOS'         'Backup VIDEOS'
```

**Attention** si vous n’avez pas un environnement graphique, certains de ces dossiers n’existent pas.
C’est pourquoi vous trouverez dans les [exemples de configuration](rsync-now-configuration-samples/)
un [petit script](rsync-now-configuration-samples/rsync-configuration-template-generator.sh)
qui tentera de générer un code plus adapter à votre configuration.

Dans l’exemple si dessus, on utilise la variable `backups_dir` (nom arbitraire). Cette variable doit être
définie quelque part, soit vous l’exporter depuis votre profile, soit vous l’ajouter dans la configuration :

Pour définir la variable `backups_dir` dans la configuration vous devrez utiliser le paramètre `--declare`.

Voici la procédure si vous prévoyez d’utiliser l’**hôte local** (dossier local ou disque USB).
Pour l’exemple disons que l’on souhaite faire la sauvegarde dans `${HOME}\my-backup-dir`

Vous devez d’abord créer ce répertoire :

```bash
mkdir -vp "${HOME}/my-backup-dir"
```

Ensuite, il vous suffit de définir une nouvelle variable (nommée `backups_dir` dans les lignes précédentes):

```bash
rsync-now-config -C DEFAULT --declare backups_dir '${HOME}/my-backup-dir'
```


Documentation concernant la configuration
-----------------------------------------

Vous devez avoir une configuration `ssh` appropriée si vous prévoyez d’utiliser cet outil pour sauvegarder sur un hôte distant.

Pour utiliser `rsync-now` vous devez faire une configuration en utilisant` rsync-now-config` :

### Utilisation de `rsync-now-config`

Outil de configuration pour rsyn-now

```
rsync-now-config --help

rsync-now-config --new LOCAL_DIR REMOTE_DIR DESCRIPTION [--config-name CONF_NAME]

rsync-now-config --set-remote       LOCAL_DIR REMOTE_DIR    [--config-name CONF_NAME]
rsync-now-config --set-description  LOCAL_DIR DESCRIPTION   [--config-name CONF_NAME]
rsync-now-config --set-enabled      LOCAL_DIR on|off        [--config-name CONF_NAME]
rsync-now-config --set-options-json LOCAL_DIR RSYNC_OPTIONS [--config-name CONF_NAME]

rsync-now-config --delete LOCAL_DIR [--config-name CONF_NAME]

rsync-now-config --declare-ssh-user    SSH_USER          [--config-name CONF_NAME]
rsync-now-config --declare-ssh-address SSH_ADDRESS       [--config-name CONF_NAME]
rsync-now-config --declare             VARNAME     VALUE [--config-name CONF_NAME]

rsync-now-config --delete-variable VARNAME [--config-name CONF_NAME]

rsync-now-config --list
rsync-now-config --list-entries   [--config-name CONF_NAME]
rsync-now-config --list-variables [--config-name CONF_NAME]

rsync-now-config --show-entry LOCAL_DIR  [--config-name CONF_NAME]
rsync-now-config --show-filename         [--config-name CONF_NAME]
rsync-now-config --show-variable VARNAME [--config-name CONF_NAME]
```

### Détails des actions possibles

| Action                                       | Description                                                        |
|----------------------------------------------|--------------------------------------------------------------------|
| `-h` ou `--help`                             | Affiche le message d’aide et quitte.                               |
| `--new LOCAL_DIR REMOTE_DIR DESCRIPTION`     | Ajoute une nouvelle entrée.                                        |
| `--set-remote LOCAL_DIR REMOTE_DIR`          | Met à jour le chemin du dossier distant (chemin de la sauvegarde) pour l’entrée `LOCAL_DIR`. |
| `--set-description LOCAL_DIR DESCRIPTION`    | Met à jour la description pour l’entrée `LOCAL_DIR`.               |
| `--set-enabled LOCAL_DIR on/off`             | Met à jour l’état d’activation par défaut pour l’entrée `LOCAL_DIR`. |
| `--set-options-json LOCAL_DIR RSYNC_OPTIONS` | Défini des options spécifiques pour **rsync** pour l’entrée `LOCAL_DIR`. |
| `--delete LOCAL_DIR`                         | Supprime toute la configuration relative à `LOCAL_DIR`.            |
| `--declare-ssh-user SSH_USER`                | Configure le nom de l’utilisateur pour le protocole ssh (paramètre commun) |
| `--declare-ssh-address SSH_ADDRESS`          | Configure l’adresse ssh du serveur distant (paramètre commun)      |
| `--declare VARNAME VALUE`                    | Ajoute ou modifie une variable définie par son nom et par sa valeur. |
| `--delete-variable VARNAME`                  | Supprime la variable à partir de son nom.                          |
| `--list`                                     | Affiche la liste des configuration disponibles                     |
| `--list-entries`                             | Afficher le nom de toutes les entrées (répertoire local).          |
| `--list-variables`                           | Afficher tous les noms variables définis.                          |
| `--show-entry LOCAL_DIR`                     | Affiche toutes les valeurs de l’entrée `LOCAL_DIR`.                |
| `--show-filename`                            | Affiche le nom complet du fichier de configuration et quitte.      |
| `--show-variable VARNAME`                    | Affiche la valeur de `VARNAME`.                                    |


### Détails des paramètres disponibles

| Paramètre                                   | Description                                                       |
|---------------------------------------------|-------------------------------------------------------------------|
| `-C CONF_NAME` ou `--config-name CONF_NAME` | Défini un espace de nom pour la configuration.                    |
| `-V` or `--verbose`                         | L’affichage sera plus verbeux                                      |

Utilisation de `rsync-now`
--------------------------

### Modes disponibles

```
  rsync-now --help
  rsync-now [--all|--enabled] [--config-name CONF_NAME] [--ignore-destination-checks]
```

### Détail des options disponibles

| Option                                      | Description                                                                      |
|---------------------------------------------|----------------------------------------------------------------------------------|
| `-h` ou `--help`                            | Affiche le message d’aide et quitte.                                             |
| `-A` ou `--all`                             | Synchronise toutes les entrées pour la configuration courante (non interactif).  |
| `-D` or `--default`                         | Synonyme de --enabled                                                            |
| `-E` or `--enabled`                         | Synchronise les entrées active pour la configuration courante (non interactif).  |
| `-C CONF_NAME` ou `--config-name CONF_NAME` | Défini un espace de nom pour la configuration.                                   |
| `-I` ou `--ignore-destination-checks`       | Permet d’effectuer la sauvegarde même si le répertoire destination n’existe pas. |


### Conseils et astuces

#### Répertoire de sauvegarde

Que ce soit localement ou sur un ordinateur distant, par défaut le répertoire de destination doit exister, si ce n’est
pas le cas l’option `--ignore-destination-checks` doit être utilisée.

#### Comment sauvegarder sur une machine distante

Vous devez définir le nom d’utilisateur pour la connexion `ssh` à l’aide de `--declare-ssh-user` et l’adresse
de la machine distante à l’aide de `--declare-ssh-address` :

```bash
rsync-now-config --declare-ssh-user    'SSH_USER'    # Optionnellement : --config-name 'CUSTOM_CONF_NAME'
rsync-now-config --declare-ssh-address 'SSH_ADDRESS' # Optionnellement : --config-name 'CUSTOM_CONF_NAME'
```

Assurez-vous que votre clé publique soit correctement configurer dans le fichier `.ssh/authorized_keys` de la machine
distante.

#### `rsync` configuration avancée

```bash
rsync-now-config -C TEST --set-options-json "${HOME}/Pictures" '{ "exclude": [ "dir1/to/ignore", "dir/to/ignore/number2" ] , "raw": ["--no-links"] }'

```

#### JSON

Le fichier de configuration se trouve sous le dossier utilisateur, le répertoire est `${HOME}/.config/rsync-now/` et
le nom du fichier par défaut est `rsync-now.json`.

Vous pouvez modifier directement le fichier en respectant le formalisme JSON.

Pour obtenir le nom du fichier de configuration (ainsi que son chemin) à partir du nom de la configuration, vous utilisez:

```bash
rsync-now-config.sh -C TEST --show-filename
```

#### Profil

Vous pouvez utiliser de nombreux profils en utilisant : `--config-name` ou` -C`.

#### Variables

Les variables SHELL (`bash`) peuvent être utilisées dans la configuration (dans le fichier JSON),
Ces variables peuvent même être définies dans la configuration.

Pour insérer une variable, vous devez utiliser le guillemet simple `'` (apostrophe):

```bash
rsync-now-config -C DEFAULT --declare some_dir '${HOME}/my-dir'
# Enregistrera :
#   "some_dir": "${HOME}/my-dir"

```

Si vous utiliser le guillemet double `"` la variable sera résolution avant la sauvegarde.

```bash
rsync-now-config -C DEFAULT --declare some_dir "${HOME}/my-dir"
# Enregistrera :
#   "some_dir": "/homes/username/my-dir"
```

