﻿rsync-now
=========

`rsync-now` is a` rsync` based tool, it allows you to set up and save your backups.

It comes with a configuration tool: `rsync-now-config`.

`rsync-now` allows you to save your data locally or on a remote server using `ssh` protocol.


Installation
------------

To download the project and perform the installation you must use the following commands:

```bash
git clone --recurse-submodules https://git.cclaude.rocks/tools/rsync-now.git
cd rsync-now
sudo bash rsync-now-install.sh
```

### Install options

You can get help using :

```bash
bash ./prepare-raspbian-sd-install.sh --help
```

| Option                            | Description                   |
|-----------------------------------|-------------------------------|
| -S, --skip-update                 | To skip update before install required packets |
| -P PMEXE, --package-manager PMEXE | Force package manager binary name |


Quick setup
-----------

First you need to create configuration using `rsync-now-config`:

```bash
rsync-now-config -new "$( xdg-user-dir DOCUMENTS )" '/path/to/external_drive' 'Save to USB Drive'
```

Simple usage:

```bash
rsync-now
```


Configuration example
---------------------

First, you need to define the folders you want to back up,
here is an example of code that should work on most Linux distributions.

```bash
rsync-now-config -C DEFAULT --new '/home/claude/Documents'   '${backups_dir}/DOCUMENTS'      'Backup DOCUMENTS'
rsync-now-config -C DEFAULT --new '/home/claude/Downloads'   '${backups_dir}/DOWNLOAD'       'Backup DOWNLOAD'
rsync-now-config -C DEFAULT --new '/home/claude/Music'       '${backups_dir}/MUSIC'          'Backup MUSIC'
rsync-now-config -C DEFAULT --new '/home/claude/Pictures'    '${backups_dir}/PICTURES'       'Backup PICTURES'
rsync-now-config -C DEFAULT --new '/home/claude/Public'      '${backups_dir}/PUBLICSHARE'    'Backup PUBLICSHARE'
rsync-now-config -C DEFAULT --new '/home/claude/Templates'   '${backups_dir}/TEMPLATES'      'Backup TEMPLATES'
rsync-now-config -C DEFAULT --new '/home/claude/Videos'      '${backups_dir}/VIDEOS'         'Backup VIDEOS'
```

**Warning** If you do not have a graphical environment, some of these folders do not exist.
This is why you will find in the [configuration examples](rsync-now-configuration-samples/)
a [small script](rsync-now-configuration-samples/rsync-configuration-template-generator.sh)
which will try to generate a code more suited to your configuration.

In the example above, we use the variable `backups_dir` (arbitrary name). This variable must be
defined somewhere, either export it from your profile, or add it to the configuration:

To set the `backups_dir` variable in the configuration you will need to use the `--declare` parameter.

Here is the procedure if you plan to use the **local host** (local folder or USB drive).
For the example let's say that we want to save in `${HOME}\my-backup-dir`

You must first create this directory:

```bash
mkdir -vp "${HOME}/my-backup-dir"
```

Then, you just need to define a new variable (named `backups_dir` in previous lines):

```bash
rsync-now-config -C DEFAULT --declare backups_dir '${HOME}/my-backup-dir'
```


Configuration documentation
---------------------------

You need to have a proper `ssh` configuration if you plan to use this tool to backup onto a remote host.

To use `rsync-now` you need to do some configuration using `rsync-now-config`:

### `rsync-now-config` usage

Configuration tool for rsync-now

```
rsync-now-config --help

rsync-now-config --new LOCAL_DIR REMOTE_DIR DESCRIPTION [--config-name CONF_NAME]

rsync-now-config --set-remote       LOCAL_DIR REMOTE_DIR    [--config-name CONF_NAME]
rsync-now-config --set-description  LOCAL_DIR DESCRIPTION   [--config-name CONF_NAME]
rsync-now-config --set-enabled      LOCAL_DIR on|off        [--config-name CONF_NAME]
rsync-now-config --set-options-json LOCAL_DIR RSYNC_OPTIONS [--config-name CONF_NAME]

rsync-now-config --delete LOCAL_DIR [--config-name CONF_NAME]

rsync-now-config --declare-ssh-user    SSH_USER          [--config-name CONF_NAME]
rsync-now-config --declare-ssh-address SSH_ADDRESS       [--config-name CONF_NAME]
rsync-now-config --declare             VARNAME     VALUE [--config-name CONF_NAME]

rsync-now-config --delete-variable VARNAME [--config-name CONF_NAME]

rsync-now-config --list
rsync-now-config --list-entries   [--config-name CONF_NAME]
rsync-now-config --list-variables [--config-name CONF_NAME]

rsync-now-config --show-entry LOCAL_DIR  [--config-name CONF_NAME]
rsync-now-config --show-filename         [--config-name CONF_NAME]
rsync-now-config --show-variable VARNAME [--config-name CONF_NAME]
```

### Actions detail

| Action                                       | Description                                                                                  |
|----------------------------------------------|----------------------------------------------------------------------------------------------|
| `-h` or `--help`                             | Displays help message and exits.                                                             |
| `--new LOCAL_DIR REMOTE_DIR DESCRIPTION`     | Add a new entry, base on local directory and a description                                   |
| `--set-remote LOCAL_DIR REMOTE_DIR`          | Updates path of remote folder (backup path) for the `LOCAL_DIR` entry.                       |
| `--set-description LOCAL_DIR DESCRIPTION`    | Updates description for the `LOCAL_DIR` entry.                                               |
| `--set-enabled LOCAL_DIR on/off`             | Updates default activation state for the `LOCAL_DIR` entry.                                  |
| `--set-options-json LOCAL_DIR RSYNC_OPTIONS` | Set specific options for **rsync** for the `LOCAL_DIR` entry (Using JSON).                   |
| `--delete LOCAL_DIR`                         | Delete all the configuration related to `LOCAL_DIR`.                                         |
| `--declare-ssh-user SSH_USER`                | Configure user name the ssh protocol (common setting).                                       |
| `--declare-ssh-address SSH_ADDRESS`          | Configure the ssh address of the remote server (common setting).                             |
| `--declare VARNAME VALUE`                    | Add or modify a variable defined by its name and its value.                                  |
| `--delete-variable VARNAME`                  | Remove the variable from its name.                                                           |
| `--list`                                     | Display available configuration names                                                        |
| `--list-entries`                             | Displays the names of all entries (local directory).                                         |
| `--list-variables`                           | Displays all defined variable names.                                                         |
| `--show-entry LOCAL_DIR`                     | Displays all values for `LOCAL_DIR` entry.                                                   |
| `--show-filename`                            | Display configuration file name and exit.                                                    |
| `--show-variable VARNAME`                    | Displays the value of `VARNAME`.                                                             |

### Parameters detail

| Parameter                                   | Description                                                       |
|---------------------------------------------|-------------------------------------------------------------------|
| `-C CONF_NAME` or `--config-name CONF_NAME` | Defines a namespace for the configuration.                        |
| `-V` or `--verbose`                         | Be more verbose                                                   |


`rsync-now` usage
-----------------

### Available modes

```
  rsync-now --help
  rsync-now [--all|--enabled] [--config-name CONF_NAME] [--ignore-destination-checks]
```

### Options descriptions

| Option                                      | Description                                                                      |
|---------------------------------------------|----------------------------------------------------------------------------------|
| `-h` or `--help`                            | Displays help and exits.                                                         |
| `-A` or `--all`                             | Synchronize all folders for current configuration (non interactive).             |
| `-D` or `--default`                         | Synonym for --enabled                                                            |
| `-E` or `--enabled`                         | Synchronize enabled folders for current configuration (non interactive).         |
| `-C CONF_NAME` or `--config-name CONF_NAME` | Defines name-space for the configuration.                                        |
| `-I` or `--ignore-destination-checks`       | Allow to run backup even if destination directory does not exists.               |


### Tips

#### Backup directory

Whether locally or on a remote computer, by default the destination directory must exist, if not
not the option `--ignore-destination-checks` must be used.

#### How to backup on a remote host

You must define the username for the `ssh` connection using` --declare-ssh-user` and the address
from the remote machine using `--declare-ssh-address`:

```bash
rsync-now-config --declare-ssh-user    'SSH_USER'    # Optionally: --config-name 'CUSTOM_CONF_NAME'
rsync-now-config --declare-ssh-address 'SSH_ADDRESS' # Optionally: --config-name 'CUSTOM_CONF_NAME'
```

Make sure your public key is correctly configured in the machine's `.ssh/authorized_keys` file
remote.

#### `rsync` advanced configuration

```bash
rsync-now-config -C TEST --set-options-json "${HOME}/Pictures" '{ "exclude": [ "dir1/to/ignore", "dir/to/ignore/number2" ] , "raw": ["--no-links"] }'

```

#### JSON

Configuration file is located under the user folder, directory is `${HOME}/.config/rsync-now/` and
the default file name is `rsync-now.json`.

You can directly edit the file using JSON formalism.

To get configuration filename (with location) from `config-name`, you should use:

```bash
rsync-now-config.sh -C TEST --show-filename
```

#### Profile

You can use many profiles using: `--config-name` or `-C`.

#### Variables

SHELL variables (`bash`) can be used in the configuration (in JSON file),
These variables can even be defined in the configuration.

To insert a variable, you must use the single quote `'` (apostrophe):

```bash
rsync-now-config -C TEST --declare some_dir '${HOME}/my-dir'
# Will record:
#   "some_dir": "${HOME}/my-dir"

```

If you use the quotation mark `"` the variable will be resolved before saving.

```bash
rsync-now-config -C TEST --declare some_dir "${HOME}/my-dir"
# Will record:
#   "some_dir": "/homes/username/my-dir"
```

