#!/bin/bash -e
#
# install-api:install-desktop-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi

function sudo_for_sudo_user_if_needed {
  if [ -n "${SUDO_USER:-}" ] ; then
    echo  sudo -u "${SUDO_USER}"
  fi
}

function install_api_xdg_user_dir_for_user {
  local -r name="${1:-}"

  $( sudo_for_sudo_user_if_needed ) xdg-user-dir "${name}" || return $?
}

function install_api_xdg_user_dir_by_name_for_user {
  local -r name="${1:-}"

  local dir_for_user=
  dir_for_user="$( install_api_xdg_user_dir_for_user "${name}" )" || return $?
  local -r dir_for_user

  if [ ! -d "${dir_for_user}" ] ; then
    echofatal "${FUNCNAME[0]}( '${name}' ) - Directory not found: '${dir_for_user}'"
    return 100
  fi
  echo "${dir_for_user}"
}


function __build_desktop_file__ {
  local -r desktop_file_template="$1"

  if [ ! -f "${desktop_file_template}" ] ; then
    echofatal "${FUNCNAME[0]}( '${desktop_file_template}' ) - File not found."
    return 100
  fi

  local _basename_=
  _basename_="$( basename "${desktop_file_template}" )" || return $?

  echo "${INSTALL_DATA_DIR}/${_basename_}"
}

function install_api_install_desktop_icon {
  local -r desktop_file_template="$1"

  local __desktop_file__=
  __desktop_file__="$( __build_desktop_file__ "${desktop_file_template}" )" || return $?
  local -r __desktop_file__

  echoinfo_quiet "${FUNCNAME[0]}( '${desktop_file_template}' ) -> '${__desktop_file__}'"

  envsubst <"${desktop_file_template}" >"${__desktop_file__}" || return $?
  desktop-file-validate "${__desktop_file__}" || return $?

# local -r dir="${INSTALL_DESKTOP_DIR:-${INSTALL_API_DEFAULT_GLOBAL_DESKTOP_DIR}}"

# (
#   set -x
#   desktop-file-install --dir="${dir}" "${__desktop_file__}"
# ) || return $?
# (
#   set -x
#   update-desktop-database "${dir}"
# ) || return $?

  local -r xdg_desktop_menu_install_mode='system'
  # Set XDG_UTILS_DEBUG_LEVEL=1 if needed
  (
    set -x
    xdg-desktop-menu install --novendor --mode "${xdg_desktop_menu_install_mode}" "${__desktop_file__}" || return $?
  ) || return $?



  # Set XDG_DEBUG_LEVEL=1 if needed
# xdg-desktop-icon install --novendor "${final_location}" || return $?
# chmod +x "${final_location}" || return $?
}

function install_api_remove_desktop_icon {
  local -r desktop_file_template="$1"

  local __desktop_file__=
  __desktop_file__="$( __build_desktop_file__ "${desktop_file_template}" )" || return $?
  local -r __desktop_file__

  echoinfo_quiet "${FUNCNAME[0]}( '${desktop_file_template}' ) -> '${__desktop_file__}'"
# xdg-desktop-icon uninstall "${final_location}" || return $?

  if [ -f "${__desktop_file__}" ] ; then
    rm -v "${__desktop_file__}" || return $?
  fi
}

function install_api_install_default_desktop_icons {
  if ! is_variable_define_safe 'INSTALL_APP_DESKTOP_ICONS' ; then
    echowarn 'INSTALL_APP_DESKTOP_ICONS is not define'
  elif [ "${#INSTALL_APP_DESKTOP_ICONS[@]}" -gt 0 ] ; then
    local file=
    for file in "${INSTALL_APP_DESKTOP_ICONS[@]}" ; do
      install_api_install_desktop_icon "${file}" || return $?
    done
  fi
}

function install_api_remove_default_desktop_icons {
  if ! is_variable_define_safe 'INSTALL_APP_DESKTOP_ICONS' ; then
    echowarn 'INSTALL_APP_DESKTOP_ICONS is not define'
  elif [ "${#INSTALL_APP_DESKTOP_ICONS[@]}" -gt 0 ] ; then
    local file=
    for file in "${INSTALL_APP_DESKTOP_ICONS[@]}" ; do
      install_api_remove_desktop_icon "${file}" || true # Ignore errors
    done
  fi
}
