#!/bin/bash
#
# ssh-api.sh:rsync-generic-main.sh
#

if [ "$( type -t ssh_api_initialize_SSH_USER_or_exit )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/ssh-common-api.sh" || exit $?
fi
if [ "$( type -t ssh_api_rsync_directory_to_host )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/rsync-api.sh" || exit $?
fi
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required." >&2
  echo "ERROR in '${BASH_SOURCE[0]}' - source message-api.sh" >&2
  exit 1
fi
if [ "$( type -t P )" != 'function' ] ; then
  echo "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required." >&2
  echo "ERROR in '${BASH_SOURCE[0]}' - source bash-common-api.sh" >&2
  exit 1
fi

function _ssh_api_rsync_generic_main_usage {
  _ssh_api_rsync_generic_main_usage_usage
  echo
  _ssh_api_rsync_generic_main_usage_description
  echo
  _ssh_api_rsync_generic_main_usage_parameters
}

function _ssh_api_rsync_generic_main_usage_usage {
  cat <<EOF
Usage:

  $( basename "$0" ) --help
  $( basename "$0" ) --source SOURCE --push-only
  $( basename "$0" ) --source SOURCE --initialize-and-push
EOF
}

function _ssh_api_rsync_generic_main_usage_description {
  cat <<EOF
Optionaly prepare source directory then push it on host ${SSH_HOST:-SSH_HOST}:${SSH_REMOTE_DIR:-SSH_REMOTE_DIR}
EOF
}

function _ssh_api_rsync_generic_main_usage_parameters {
  cat <<EOF
  -h,   --help              Display help and exit.
  -I,   --initialize-and-push
                            Prepare data before pushing to server.
  -P,   --push-only         Do not convert before pushing
  -S,   --source            Set source directory
EOF
}

function _ssh_api_rsync_do_push_directory_to_host {
  if [ "${MESSAGE_API_QUIET:-}" != 'true' ] ; then
    echoinfo_quiet "Update host: '${SSH_HOST}' using '${SSH_USER}'"
    echoinfo_quiet "Copy from: '${param_source_directory}'"
    echoinfo_quiet "to: '${SSH_REMOTE_DIR}'"
    ssh_api_RSYNC_OPTS_ARRAY_show_opts || return $?
  fi

  ssh_api_rsync_directory_to_host "${param_source_directory}" \
    "${SSH_USER}" "${SSH_HOST}" "${SSH_REMOTE_DIR}" || return $?
}

function ssh_api_rsync_generic_main {
  local param=
  local param_initialize=
  local param_push='false'
  local param_source_directory=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        _ssh_api_rsync_generic_main_usage
        return 0
        ;;

      '-I'|'--initialize-and-push'|'--initialiaze')
        param_initialize='true'
        param_push='true'
        ;;

      '--initialiaze-and-push')
        echowarn 'Please fix typo use --initialize-and-push'
        param_initialize='true'
        param_push='true'
        ;;

      '-P'|'--push-only')
        param_initialize='false'
        param_push='true'
        ;;

      '-S'|'--source')
        if ! param_source_directory="$( P "$@" )" ; then
          echofatal '--source require directory path'
          _ssh_api_rsync_generic_main_usage >&2
          return 100
        fi
        shift
        ;;

      *)
        echofatal "Unknown option '${param}'."
        _ssh_api_rsync_generic_main_usage >&2
        return 100
        ;;
    esac
  done

  if [ -z "${param_initialize}" ] ; then
    echofatal 'You must use --push-only or --convert-and-push'
    _ssh_api_rsync_generic_main_usage >&2
    return 100
  fi
  if [ -z "${param_source_directory}" ] ; then
    echofatal 'You must define --source parameter'
    _ssh_api_rsync_generic_main_usage >&2
    return 100
  fi
  if [ ! -d "${param_source_directory}" ] ; then
    echofatal "Directory not found: '${param_source_directory}'."
    _ssh_api_rsync_generic_main_usage >&2
    return 100
  fi

  if [ "${param_initialize}" = 'true' ] ; then
    if [ "$( type -t do_initialize )" != 'function' ] ; then
      echofatal 'Function do_initialize is required'
      return 100
    fi

    do_initialize || return $?
  fi

  if [ "${param_push}" = 'true' ] ; then
    (
      ssh_api_initialize_SSH_USER_or_exit "${SSH_HOST}"
      _ssh_api_rsync_do_push_directory_to_host || return $?
    ) || return $?
  fi
}
