#!/bin/bash
#
# ssh-api:scp-api.sh
#
# NOTE: scp is deprecated on Linux in favor of rsync.
# In some cases you may also consider using sshfs.
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required." >&2
  echo "ERROR in '${BASH_SOURCE[0]}' - source message-api.sh" >&2
  exit 1
fi

echowarn 'ssh-api:scp-api.sh - scp is deprecated on Linux in favor of rsync.' # DEPRECATED 2024-01-14

function ssh_api_scp_to_host {
  local -r _local_file_="$1"
  local -r _ssh_user_="$2"
  local -r _ssh_host_="$3"
  local -r _ssh_remote_file_="$4"

  if [ ! -f "${_local_file_}" ] ; then
    echofatal "${FUNCNAME[0]}() - Local file not found: '${_local_file_}'"
    return 100
  fi

  (
    if [ "${SSH_API_SCP_DEBUG:-}" = 'true' ] ; then
      set -x
    fi
    # shellcheck disable=SC2086 # (info): Double quote to prevent globbing and word splitting.
    scp ${SSH_API_SCP_OPTS:-} "${_local_file_}" "${_ssh_user_}@${_ssh_host_}:${_ssh_remote_file_}"
  ) || return $?
}

function ssh_api_scp_from_host {
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_remote_file_="$3"
  local -r _local_file_="$4"

  (
    if [ "${SSH_API_SCP_DEBUG:-}" = 'true' ] ; then
      set -x
    fi
    # shellcheck disable=SC2086 # (info): Double quote to prevent globbing and word splitting.
    scp ${SSH_API_SCP_OPTS:-}  "${_ssh_user_}@${_ssh_host_}:${_ssh_remote_file_}" "${_local_file_}"
  ) || return $?
}

function ssh_api_scp_directory_to_host {
  local -r _local_directory_="$1"
  local -r _ssh_user_="$2"
  local -r _ssh_host_="$3"
  local -r _ssh_remote_dir_="$4"

  if [ ! -d "${_local_directory_}" ] ; then
    echofatal "${FUNCNAME[0]}() - Local directory not found: '${_local_directory_}'"
    return 100
  fi

  (
    if [ "${SSH_API_SCP_DEBUG:-}" = 'true' ] ; then
      set -x
    fi
    # shellcheck disable=SC2086 # (info): Double quote to prevent globbing and word splitting.
    scp -rp ${SSH_API_SCP_OPTS:-} "${_local_directory_}" "${_ssh_user_}@${_ssh_host_}:${_ssh_remote_dir_}"
  ) || return $?
}
