#!/bin/bash
#
# ssh-api:ssh-api.sh
#
if [ "$( type -t ssh_api_SSH_OPTS_ARRAY_set_opts )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/ssh-config-api.sh" || exit $?
fi
if [ "$( type -t __ssh_api_common_private_get_ssh_host_ip__ )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/ssh-common-api.sh" || exit $?
fi

function ssh_api_do_ssh { # was: _ssh_api_do_ssh / __ssh_api_private_do_ssh__
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_cmd_="$3"

  local _ssh_ip_host_=
  _ssh_ip_host_="$( __ssh_api_common_private_get_ssh_host_ip__ "${_ssh_host_}" )" || return $?
  local -r _ssh_ip_host_

  if [ "${__SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT_INIT__}" != 'true' ] ; then
    local _ssh_opts_=
    _ssh_opts_="$( __ssh_api_private_get_SSH_OPTS__ )" || return $?
    local -r _ssh_opts_

    ssh_api_SSH_OPTS_ARRAY_set_opts_from_string "${_ssh_opts_}" || return $?
  fi

  local -i ssh_rc=0

  (
    if ssh_api_is_SSH_API_SSH_DEBUG ; then
      set -x
    fi

    # shellcheck disable=SC2029 # _ssh_cmd_ (info): Note that, unescaped, this expands on the client side.
    ssh "${__SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT__[@]}" "${_ssh_user_}@${_ssh_ip_host_}" "${_ssh_cmd_}"
  ) || ssh_rc=$?

  if [ "${ssh_rc}" -ne 0 ] ; then
    echofatal "${FUNCNAME[0]}() - ssh error:  ${ssh_rc} running with '${USER:-$( id -u -n )}' from: '$( pwd )'"
    echofatal "${FUNCNAME[0]}() - USER: '${USER:-}' / id -u -n: '$( id -u -n )'"
    echofatal "ssh access: '${_ssh_user_}@${_ssh_ip_host_}'"
    echofatal "ssh resolution: _ssh_ip_host_='${_ssh_ip_host_}'"
    echofatal "ssh resolution: _ssh_host_='${_ssh_host_}'"
    echofatal "SSH_AGENT_PID=${SSH_AGENT_PID:-}"
    echofatal "SSH_AUTH_SOCK=${SSH_AUTH_SOCK:-}"
    echofatal "ssh command: ${_ssh_cmd_}"
    echofatal "stacktrace: ${FUNCNAME[*]}"

    if ssh_api_is_SSH_API_SSH_DEBUG ; then
      echofatal 'env'
      ssh_api_print_env >&2
    fi

    return "${ssh_rc}"
  fi
}

function ssh_api_test_remote_directory {
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_remote_dir_="$3"

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug "${FUNCNAME[0]}() - _ssh_user_='${_ssh_user_}'"
    echodebug "${FUNCNAME[0]}() - _ssh_host_='${_ssh_host_}'"
    echodebug "${FUNCNAME[0]}() - _ssh_remote_dir_='${_ssh_remote_dir_}'"
  fi

  ssh_api_do_ssh "${_ssh_user_}" "${_ssh_host_}" \
    "if [ -d '${_ssh_remote_dir_}' ] ; then echo 'FOUND' ; else echo 'NOTEXISTS' ; fi" || return $?
}

