#!/bin/bash
#
# ssh-api:ssh-config-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required." >&2
  echo "ERROR in '${BASH_SOURCE[0]}' - source message-api.sh" >&2
  exit 100
fi

declare -ga __SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT__=()
declare -g  __SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT_INIT__='false'

function __ssh_api_private_get_SSH_OPTS__ {
  if [ -n "${SSH_API_SSH_OPTS:-}" ] ; then
    echowarn "Use of SSH_API_SSH_OPTS is deprecated. Use ssh_api_SSH_OPTS_ARRAY_set_opts_from_string() instead."
    echo "${SSH_API_SSH_OPTS}"
  else
    #
    # https://stackoverflow.com/questions/16045547/why-ssh-breaks-the-control-flow-in-bash
    #
    # -n Redirects stdin from /dev/null (actually, prevents reading from stdin).
    #
    echo '-n'
  fi
}

function ssh_api_get_SSH_OPTS { # DEPRECATED 2023-01-13
  if [ -n "${SSH_API_SSH_OPTS:-}" ] ; then
    echowarn "${FUNCNAME[0]}() is deprecated. Use of SSH_API_SSH_OPTS is deprecated."
    echo "${SSH_API_SSH_OPTS}"
  else
    echowarn "${FUNCNAME[0]}() is deprecated."
    echo '-n'
  fi
}

function ssh_api_SSH_OPTS_ARRAY_add_opts_from_string {
  local -r opts_str="$1"

  IFS=' ' read -r -a array <<<"${opts_str}" || return $?

  ssh_api_SSH_OPTS_ARRAY_add_opts "${array[@]}" || return $?
}

function ssh_api_SSH_OPTS_ARRAY_add_opts {
  declare -g __SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT_INIT__='true'

  if [ "$#" -gt 0 ] ; then
    local v=
    for v in "$@" ; do
      __SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT__+=("${v}")
    done
  fi
}

function ssh_api_SSH_OPTS_ARRAY_set_opts_from_string {
  local -r opts_str="$1"

  declare -ga __SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT__=()

  ssh_api_SSH_OPTS_ARRAY_add_opts_from_string "${opts_str}" || return $?
}

function ssh_api_SSH_OPTS_ARRAY_set_opts {
  declare -ga __SSH_API_PRIVATE_SSH_OPTS_ARRAY_STRICT__=()

  ssh_api_SSH_OPTS_ARRAY_add_opts "$@" || return $?
}
